#include "rlft3.h"

const double Pi = 3.1415926535897932385;

//======================================================================
//Parameters
const double Lx = 6;
const double Ly = 6;

const double Lx_frame = 1.5; // distance from end to box
const double Ly_frame = 1.5;

const double Lx_fixed = 1.6;
const double Ly_fixed = 1.6;

const int Fixed_Actin = 2*100;

const int NumXPoints = 128;
const int NumYPoints = 128;

double dx = Lx/(NumXPoints);
double dy = Ly/(NumYPoints);

int NumActin = 3*4000;
const int NumMyosin = 3*4000;
int AvgNumActin = NumActin;
const int MaxNumActin = 10 * NumActin;

const double nu_d = .1; // destruction rate of actin particles
const double mu_att = .1;//.1; // attachment rate for myosin particles per second per actin particle
const double mu_att_strip = 1; // attachement rate of myosin in a strip

const double D = .001;
const double Dmu = .001;
const double Dmu_f = .01;
const double alpha = 2.0*.1*dx*dy;

const double eps = 1.0e-3;



const int NumFrames = 200; // How many frames does the movie have

int Frame_Count = 1;

const double dt = 0.001*2*2;//*10*4/2/2/2;
//======================================================================

class GridData
{
public:
	GridData();

	double * operator[] (int i) {return f[1][i];}
	double * s() {return f_speq[1];}

	void operator *= (double f);

	void operator += (GridData * GD);

	void operator = (GridData * GD);

	double *** get_f() {return f;};
	double ** get_f_speq() {return f_speq;};

private:
	double *** f; // this is a tensor
	double ** f_speq; // this is a matrix
};

void GridData::operator += (GridData * GD)
{
	int i=0, j=0;

	for(i=1; i<=NumXPoints; i++)
	{
		for(j=1; j<=NumYPoints; j++)
		{
			f[1][i][j] += (*GD)[i][j];
		}
	}

	for(i=1; i<=2*NumXPoints; i++)
	{
		f_speq[1][i] += GD->s()[i];
	}
}

GridData::GridData()
{
	f  = f3tensor(1,1,1,NumXPoints,1,NumYPoints);
	f_speq=matrix(1,1,1,2*NumXPoints);
}

void GridData::operator *=(double fl)
{
	int i=0, j=0;

	for(i=1; i<=NumXPoints; i++)
	{
		for(j=1; j<=NumYPoints; j++)
		{
			f[1][i][j] *= fl;
		}
	}

	for(i=1; i<=2*NumXPoints; i++)
	{
		f_speq[1][i] *= fl;
	}
}

void GridData::operator =(GridData * GD)
{
	int i=0, j=0;

	for(i=1; i<=NumXPoints; i++)
	{
		for(j=1; j<=NumYPoints; j++)
		{
			f[1][i][j] = (*GD)[i][j];
		}
	}

	for(i=1; i<=2*NumXPoints; i++)
	{
		f_speq[1][i] = GD->s()[i];
	}
}

void fft( GridData* GD )
{
	int i=0, j=0;

	rlft3(GD->get_f(),GD->get_f_speq(),1,NumXPoints,NumYPoints,1);

	for(i=1; i<=NumXPoints; i++)
	{
		for(j=1; j<=NumYPoints; j++)
		{
			(*GD)[i][j] /= (1*NumXPoints*NumYPoints/2);
		}
	}
	for(i=1; i<=2*NumXPoints; i++)
	{
		GD->s()[i] /= (1*NumXPoints*NumYPoints/2);
	}
}


void invfft(GridData* GD)
{
	rlft3(GD->get_f(),GD->get_f_speq(),1,NumXPoints,NumYPoints,-1);
}

void nabla2Fourier( GridData * GD )
{
	int i=0, j=0;

	for(i=0; i<=NumXPoints/2-1; i++)
	{
		for(j=0; j<=NumYPoints/2-1; j++)
		{
			(*GD)[i+1][2*j+1] *= -( i*i*pow(2*Pi/Lx,2) + j*j*pow(2*Pi/Ly,2) );
			(*GD)[i+1][2*j+2] *= -( i*i*pow(2*Pi/Lx,2) + j*j*pow(2*Pi/Ly,2) );
			
			if(i>0)
			{
				(*GD)[NumXPoints-i+1][2*j+1] *= -( i*i*pow(2*Pi/Lx,2) + j*j*pow(2*Pi/Ly,2) );
				(*GD)[NumXPoints-i+1][2*j+2] *= -( i*i*pow(2*Pi/Lx,2) + j*j*pow(2*Pi/Ly,2) );
			}
		}
	}
	//
	i=NumXPoints/2;
		for(j=0; j<=NumYPoints/2-1; j++)
		{
			(*GD)[i+1][2*j+1] *= -( i*i*pow(2*Pi/Lx,2) + j*j*pow(2*Pi/Ly,2) );
			(*GD)[i+1][2*j+2] *= -( i*i*pow(2*Pi/Lx,2) + j*j*pow(2*Pi/Ly,2) );
		}
	//
	
	for(i=0; i<=NumXPoints/2-1; i++)
	{
		GD->s()[2*i+1] *= -( (NumYPoints/2)*(NumYPoints/2)*pow(2*Pi/Ly,2)+i*i*pow(2*Pi/Lx,2) );
		GD->s()[2*i+2] *= -( (NumYPoints/2)*(NumYPoints/2)*pow(2*Pi/Ly,2)+i*i*pow(2*Pi/Lx,2) );
		
		if(i>0)
		{
			GD->s()[2*NumXPoints-2*i+1] *= -( (NumYPoints/2)*(NumYPoints/2)*pow(2*Pi/Ly,2)+i*i*pow(2*Pi/Lx,2) );
			GD->s()[2*NumXPoints-2*i+2] *= -( (NumYPoints/2)*(NumYPoints/2)*pow(2*Pi/Ly,2)+i*i*pow(2*Pi/Lx,2) );
		}
	}

	//
	i = NumXPoints/2;

		GD->s()[2*i+1] *= -( (NumYPoints/2)*(NumYPoints/2)*pow(2*Pi/Ly,2)+i*i*pow(2*Pi/Lx,2) );
		GD->s()[2*i+2] *= -( (NumYPoints/2)*(NumYPoints/2)*pow(2*Pi/Ly,2)+i*i*pow(2*Pi/Lx,2) );
	//
}

void convFourier(GridData* GD1, GridData* GD2, GridData* Result)
{
	int i=0, j=0;

	double a=0, b=0, c=0, d=0;

	for(i=1; i<=NumXPoints; i++)
	{
		for(j=1; j<NumYPoints; j+=2)
		{
			a = (*GD1)[i][j];
			b = (*GD1)[i][j+1];
			c = (*GD2)[i][j];
			d = (*GD2)[i][j+1];

			(*Result)[i][j] = a*c-b*d;
			(*Result)[i][j+1] = a*d+b*c;
		}
	}
	for(i=1; i<NumXPoints*2; i+=2)
	{
		a = GD1->s()[i];
		b = GD1->s()[i+1];
		c = GD2->s()[i];
		d = GD2->s()[i+1];

		Result->s()[i] =   a*c-b*d;
		Result->s()[i+1] = a*d+b*c;
	}

	for(i=1; i<=NumXPoints; i++)
	{
		for(j=1; j<NumYPoints; j+=2)
		{
			(*Result)[i][j] *= Lx*Ly/2;
			(*Result)[i][j+1] *= Lx*Ly/2;
		}
	}
	for(i=1; i<NumXPoints*2; i+=2)
	{
		Result->s()[i] *= Lx*Ly/2;
		Result->s()[i+1] *= Lx*Ly/2;
	}
}


char* itoa(int val, int base) {
     static char buf[32] = {0};
     int i = 30;
     	if(val==0)
     	{
     	buf[i]='0';
	return &buf[i];
	}
     for (; val && i; --i, val /= base)
         buf[i] = "0123456789abcdef"[val % base];
     return &buf[i+1];
}

