#include <iostream>
#include <fstream>
#include <math.h>
using namespace std;

#include "accessories.h"
#include "auxlrs.h"

int main()
{
	//=========================================================================================================================
	//
	int i=0, j=0, k=0, t=0;
	
	/* GridData: an object holding an array 
	 implementing FFT
	 */
	
	const double D_step = sqrt(2*D*dt);
	const double Dmu_step = sqrt(2*Dmu*dt);
	const double Dmu_f_step = sqrt(2*Dmu_f*dt);
	
	
	
	double Ax[MaxNumActin]; // position of actin
	double Ay[MaxNumActin];
	
	double dAx[MaxNumActin]; // movement of the actin
	double dAy[MaxNumActin]; // 
	
	double Mx[NumMyosin]; // position of myosin
	double My[NumMyosin];
	
	double dMx[NumMyosin]; //movement of the myosin
	double dMy[NumMyosin];
	
	int rho_dett[NumXPoints][NumYPoints]; // 1 if decay occured in box, 0 if not
	
	
	
	int FreeM[NumMyosin]; // 0 if myosin is free 1 if its not free
	
	
	double P_d = 1 - exp(-nu_d*dt);
	double P_mu_att = 1 - exp(-mu_att*dt);
	double P_mu_att_strip = 1 - exp(-mu_att_strip*dt);
	double drho_for_N = 1/dx*1/dy;
	
	double Total_Myosin = 0;
	
	long * idum = new long;
	*(idum) = 100;
	
	GridData rho; // actin density
	GridData mu; // bound myosin density
	GridData mu_f; // free myosin, not used
	GridData T; // Total number of binding sites available for binding
	GridData muT; // an accessory object for evaluating the flux of myosin
	
	// corresponding fourier trasnforms
	GridData rho_hat;
	GridData mu_hat;
	GridData T_hat;
	GridData muT_hat;

	
	GridData Phix; // Force density
	GridData Phiy;
	
	GridData Jmux;
	GridData Jmuy;
	
	GridData J1x;
	GridData J1y;
	
	GridData J2x;
	GridData J2y;
	
	GridData P; // Probability distribution of motor arm displacement
	GridData fx; // force distribution, x-component
	GridData fy; // force distribution, y-component
	
	GridData P_hat;
	
	GridData Pfx;
	GridData Pfy;
	
	GridData Pfx_hat;
	GridData Pfy_hat;
	
	GridData xs; // Actual position of the grid
	GridData ys; // Actual position of the grid
	//=========================================================================================================================
	
	//=========================================================================================================================
	//
	
	for(i=1; i<=NumXPoints; i++)
	{
		for(j=1; j<=NumYPoints; j++)
		{
			xs[i][j] = (i-1)*dx;
			ys[i][j] = (j-1)*dy;
		}
	}
	
	// initialize distribution of motor arm displacement and force density
	for(i=1; i<=NumXPoints; i++)
	{
		for(j=1; j<=NumYPoints; j++)
		{
			double x = xs[i][j];
			if(x>Lx/2) x += -Lx;
			double y = ys[i][j];
			if(y>Ly/2) y += -Ly;
			
			P[i][j] = 1.0*exp( -(pow(x,2)+pow(y,2))/pow(5.0/8,2) );
			fx[i][j] = 100.0*x*exp( -(pow(x,2)+pow(y,2))/pow(5.0/8,2) );
			fy[i][j] = 100.0*y*exp( -(pow(x,2)+pow(y,2))/pow(5.0/8,2) );
			Pfx[i][j] = P[i][j]*fx[i][j];
			Pfy[i][j] = P[i][j]*fy[i][j];
		}
	}
	
	P_hat = &P;
	Pfx_hat = &Pfx;
	Pfy_hat = &Pfy;
	
	fft(&P_hat);
	fft(&Pfx_hat);
	fft(&Pfy_hat);
	//=========================================================================================================================
	
	//=========================================================================================================================
	// Initial conditions
	for (i=0; i<MaxNumActin; i++) {
		Ax[i] = -1;
		Ay[i] = -1;
	}
	
	// for 2 diagonal peaks
	for (i=0; i<NumActin; i++){
		if (i < Fixed_Actin/2) {
			Ax[i] = Lx_fixed;
			Ay[i] = Ly_fixed;
		}
		else if ((i >= Fixed_Actin/2)&&(i < Fixed_Actin)){
			Ax[i] = Lx - Lx_fixed;
			Ay[i] = Ly - Ly_fixed;
		}
		else {
			double r = ran2(idum);
			Ax[i] = (Lx - 2*Lx_frame)*r + Lx_frame;
			Ay[i] = (Ly - 2*Ly_frame)*r + Ly_frame;
			
		}
	} 
	
	// for 4 peaks
	/*for (i=0; i<NumActin; i++){
		if (i < Fixed_Actin/4) {
			Ax[i] = Lx_fixed;
			Ay[i] = Ly_fixed;
		}
		else if ((i >= Fixed_Actin/4)&&(i < Fixed_Actin/2)){
			Ax[i] = Lx - Lx_fixed;
			Ay[i] = Ly - Ly_fixed;
		}
		else if ((i >= Fixed_Actin/2)&&(i < 3*Fixed_Actin/4)){
			Ax[i] = Lx_fixed;
			Ay[i] = Ly - Ly_fixed;
		}
		else if ((i >= 3*Fixed_Actin/4)&&(i < Fixed_Actin)){
			Ax[i] = Lx - Lx_fixed;
			Ay[i] = Ly_fixed;
		}
		else {
			Ax[i] = (Lx - 2*Lx_frame)*ran2(idum) + Lx_frame;
			Ay[i] = (Ly - 2*Ly_frame)*ran2(idum) + Ly_frame;

		}
	} */
	
	// for 5 peaks
	/*for (i=0; i<NumActin; i++){
		if (i < Fixed_Actin/5) {
			Ax[i] = Lx_fixed;
			Ay[i] = Ly_fixed;
		}
		else if ((i >= Fixed_Actin/5)&&(i < 2*Fixed_Actin/5)){
			Ax[i] = Lx - Lx_fixed;
			Ay[i] = Ly - Ly_fixed;
		}
		else if ((i >= 2*Fixed_Actin/5)&&(i < 3*Fixed_Actin/5)){
			Ax[i] = Lx_fixed;
			Ay[i] = Ly - Ly_fixed;
		}
		else if ((i >= 3*Fixed_Actin/5)&&(i < 4*Fixed_Actin/5)){
			Ax[i] = Lx - Lx_fixed;
			Ay[i] = Ly_fixed;
		}
		else if ((i >= 4*Fixed_Actin/5)&&(i < Fixed_Actin)){
			Ax[i] = Lx/2;
			Ay[i] = Ly/2;
		}
		else {
			Ax[i] = (Lx - 2*Lx_frame)*ran2(idum) + Lx_frame;
			Ay[i] = (Ly - 2*Ly_frame)*ran2(idum) + Ly_frame;
			
		}
	} */
	
	for (j=0; j<NumMyosin; j++){
		
		/*double tmp = ran2(idum);
		
		if (tmp > .5){
			Mx[j] = Lx - Lx_frame - (Lx - 2*Lx_frame)/2*sqrt(tmp*2 - 1);
		}
		else {
			Mx[j] = Lx_frame + (Lx - 2*Lx_frame)/2*sqrt(tmp*2);
		}
		
		tmp = ran2(idum);
		
		if (tmp > .5){
			My[j] = Ly - Ly_frame - (Ly - 2*Ly_frame)/2*sqrt(tmp*2 - 1);
		}
		else {
			My[j] = Ly_frame + (Ly - 2*Ly_frame)/2*sqrt(tmp*2);
		} */ 
		
		double r = ran2(idum);
		
		Mx[j] = (Lx - 2*Lx_frame)*r + Lx_frame;
		My[j] = (Ly - 2*Ly_frame)*r + Ly_frame;
		FreeM[j] = 1; // All myosin is initially free
	}
	//=========================================================================================================================
	
	//create the file for output
	
	ofstream* file = new ofstream("Dataout.m");
	(*file)<<"Lx = "<<Lx<<";"<<endl;
	(*file)<<"Ly = "<<Ly<<";"<<endl;
	(*file)<<"NumPointsX = "<<NumXPoints<<";"<<endl;
	(*file)<<"NumPointsY = "<<NumYPoints<<";"<<endl;
	(*file)<<"D = "<<D<<";"<<endl;
	(*file)<<"Dmu = "<<Dmu<<";"<<endl;
	(*file)<<"Dmu_f = "<<Dmu_f<<";"<<endl;
	(*file)<<"Alpha = "<<alpha<<";"<<endl;
	(*file)<<"Beta = "<<nu_d<<";"<<endl;
	(*file)<<"Mu_att = "<<mu_att<<";"<<endl;
	(*file)<<"dx = "<<dx<<";"<<endl;
	(*file)<<"dy = "<<dy<<";"<<endl;
	(*file)<<"dt = "<<dt<<";"<<endl;
	
	(*file)<<"Rho_array = zeros("<<NumXPoints<<", "<<NumYPoints<<", "<<NumFrames<<");"<<endl;
	(*file)<<"Mu_array = zeros("<<NumXPoints<<", "<<NumYPoints<<", "<<NumFrames<<");"<<endl;
	(*file)<<"Mu_f_array = zeros("<<NumXPoints<<", "<<NumYPoints<<", "<<NumFrames<<");"<<endl;
	
	(*file)<<"Time_Array = [];"<<endl;
	
	file->close();
	delete file;
	
	
	
	
	//=========================================================================================================================
	//=========================================================================================================================
	
	for(t=0; t < 81001;t++)
	{
		if( !(t/100.0-t/100) ) { cout<<"Iteration # "<<t<<" Time = "<<t*dt<< " Total Myosin = " << Total_Myosin <<endl;
		}
		
		Total_Myosin = 0;
		
		// initialize the density profiles to 0
		for (i=1; i<=NumXPoints; i++) {
			for (j=1; j<=NumYPoints; j++) {
				rho[i][j] = 1.0/25;
				mu[i][j] = 1.0/25;
				mu_f[i][j] = 0;
				rho_dett[i-1][j-1] = 0;
			}
		}
		
		//cout << "The error is for t = " << t << endl;
		
		// create the density profile for actin
		for (i=0; i<NumActin; i++)
			rho[int(Ax[i]/dx) + 1][int(Ay[i]/dy) + 1] += 1;
		
		/*for (j=0; j<NumMyosin; j++)
		 mu[int(Mx[j]/dx) + 1][int(My[j]/dy) + 1] += 1; */
		
		
		for (i=0;Ax[i] != -1; i++)
		{
			if ((ran2(idum) < P_d)&&(i >= Fixed_Actin))
			{
				// If this actin decays
				rho[int(Ax[i]/dx) + 1][int(Ay[i]/dy) + 1] -= 1; // remove this actin from the density
				rho_dett[int(Ax[i]/dx)][int(Ay[i]/dy)] = 1; // set flag that element decayed
				Ax[i] = Ax[NumActin-1]; // remove this position and replace it with substitute
				Ay[i] = Ay[NumActin-1];
				Ax[NumActin-1] = -1; // set the substitute to -1
				Ay[NumActin-1] = -1;
				NumActin--; // lower actin number by 1
				i--; // lower count by 1
			}
			
		}
		
		// Create New Actin
		for (i=0; i<AvgNumActin; i++) {
			if (ran2(idum) < P_d) { // if an element is created
				Ax[NumActin] = (Lx - 2*Lx_frame)*ran2(idum) + Lx_frame; // generate a random position
				Ay[NumActin] = (Ly - 2*Ly_frame)*ran2(idum) + Ly_frame;
				rho[int(Ax[NumActin]/dx) + 1][int(Ay[NumActin]/dy) + 1] += 1; // update the density profile
				NumActin++; // increase the number of actin by 1
			}
		}
		
		//cout << "The Number of Actin is " << NumActin << endl;
		
		
		// create the density profile for myosin 
		double d=0;
		
		for (j=0; j<NumMyosin; j++)
		{
			d = sqrt(Mx[j] - My[j])*1.4142;
			if (d < .1)
			{
				if (rho_dett[int(Mx[j]/dx)][int(My[j]/dy)] > 0) // if a dettachemnt occured in this spot
					if (ran2(idum) < 1/(rho[int(Mx[j]/dx) + 1][int(My[j]/dy) + 1] + 1)) // if myosin dettached
						FreeM[j] = 1; // Free this myosin
				
				// If myosin dettached and there is actin present
				if ((FreeM[j] > 0)&&(rho[int(Mx[j]/dx) + 1][int(My[j]/dy) + 1] > 0))
					if (ran2(idum) < rho[int(Mx[j]/dx) + 1][int(My[j]/dy) + 1]*P_mu_att_strip) // if it attached
						FreeM[j] = 0; // make myosin bound
				
				if (FreeM[j] == 0) // Count only the myosin that is bound
					mu[int(Mx[j]/dx) + 1][int(My[j]/dy) + 1] += 1;
			} 
			else
			{
				if (rho_dett[int(Mx[j]/dx)][int(My[j]/dy)] > 0) // if a dettachemnt occured in this spot
					if (ran2(idum) < 1/(rho[int(Mx[j]/dx) + 1][int(My[j]/dy) + 1] + 1)) // if myosin dettached
						FreeM[j] = 1; // Free this myosin
				
				// If myosin dettached and there is actin present
				if ((FreeM[j] > 0)&&(rho[int(Mx[j]/dx) + 1][int(My[j]/dy) + 1] > 0))
					if (ran2(idum) < rho[int(Mx[j]/dx) + 1][int(My[j]/dy) + 1]*P_mu_att) // if it attached
						FreeM[j] = 0; // make myosin bound
				
				if (FreeM[j] == 0) // Count only the myosin that is bound
					mu[int(Mx[j]/dx) + 1][int(My[j]/dy) + 1] += 1;
			
			}
		} 
		
		for(j=1; j<=NumXPoints; j++)
		{
			for(k=1; k<=NumYPoints; k++)
			{
				Total_Myosin += mu[j][k] - 1.0/25;
				mu[j][k] *= drho_for_N;
				rho[j][k] *= drho_for_N;
			}
		}
	
		for (j=0; j<NumMyosin; j++) {
			if (FreeM[j] == 1) {
				mu_f[int(Mx[j]/dx) + 1][int(My[j]/dy) + 1] += 1;
			}
		}
		
		// Take the fourier transform of the density profiles
		// for both Actin and Myosin
		rho_hat = &rho; // Actin
		mu_hat = &mu; // Myosin
		fft(&rho_hat); // fft: fast Fourier transform
		fft(&mu_hat);
		
		convFourier(&P_hat,&rho_hat,&T_hat); // Convolve P with rho in Fourier space
		T = &T_hat;
		invfft(&T); // Invert Fourier transform
		
		for(j=1; j<=NumXPoints; j++)
		{
			for(k=1; k<=NumYPoints; k++)
			{
				muT[j][k] = mu[j][k]/(T[j][k]+eps);
			}
		}
		muT_hat = &muT;
		fft(&muT_hat);
		
		convFourier(&Pfx_hat,&rho_hat,&J1x);
		convFourier(&Pfy_hat,&rho_hat,&J1y);
		
		convFourier(&Pfx_hat,&muT_hat,&J2x);
		convFourier(&Pfy_hat,&muT_hat,&J2y);
		
		invfft(&J1x);
		invfft(&J1y);
		invfft(&J2x);
		invfft(&J2y);
		
		for(j=1; j<=NumXPoints; j++)
		{
			for(k=1; k<=NumYPoints; k++)
			{
				J1x[j][k] *= -mu[j][k]/(eps+T[j][k])/(eps+rho[j][k]); // add epsilon to take care of T=0
				J1y[j][k] *= -mu[j][k]/(eps+T[j][k])/(eps+rho[j][k]);
				
				J2x[j][k] *= -1.0;
				J2y[j][k] *= -1.0;
			}
		}
		
		// Finally calculate the force
		for(j=1; j<=NumXPoints; j++)
		{
			for(k=1; k<=NumYPoints; k++)
			{
				Phix[j][k] = alpha*(J1x[j][k]+J2x[j][k]);
				Phiy[j][k] = alpha*(J1y[j][k]+J2y[j][k]);
				
				
			}
		}
		
		/*
		 // Move the particles
		 double ***tmpvarx;
		 double ***tmpvary;
		 
		 
		 tmpvarx = Phix.get_f();
		 tmpvary = Phiy.get_f();
		 
		 
		 
		 for (i=0; i<NumActin; i++) {
		 int Tmpx = Ax[i]/dx;
		 int Tmpy = Ay[i]/dy;
		 
		 Ax[i] += tmpvarx[1][Tmpx + 1][Tmpy + 1]*dx*dy;
		 Ay[i] += tmpvary[1][Tmpx + 1][Tmpy + 1]*dx*dy;
		 } 
		 
		 for (j=0; j<NumMyosin; j++){
		 int Tmpx = Mx[j]/dx;
		 int Tmpy = My[j]/dy;
		 
		 Mx[j] += tmpvarx[1][Tmpx + 1][Tmpy + 1]*dx*dy;
		 My[j] += tmpvary[1][Tmpx + 1][Tmpy + 1]*dx*dy;
		 } 
		 */
		
		for (i=0; i<NumActin; i++)
		{
			int Tmpx = Ax[i]/dx;
			int Tmpy = Ay[i]/dy;
			
			dAx[i]=Phix[Tmpx + 1][Tmpy + 1]*dt;
			dAy[i]=Phiy[Tmpx + 1][Tmpy + 1]*dt;
		}
		
		for (j=0; j<NumMyosin; j++) 
		{
			int Tmpx = Mx[j]/dx;
			int Tmpy = My[j]/dy;
			
			dMx[j]=Phix[Tmpx + 1][Tmpy + 1]*dt;
			dMy[j]=Phiy[Tmpx + 1][Tmpy + 1]*dt;
		}
		
		
		// Move by diffusing halfstep and implement periodic Bdy condition
		
		
		for (j=0; j<NumMyosin; j++) {
			if (Mx[j] < 0 || Mx[j] > Lx) {
				cout << "Before Out of bounds error" << endl;
				cout << "Mx = " << Mx[j] << "  dMx = " << dMx[j]<<endl;
			}
			
			if (My[j] < 0 || My[j] > Ly) {
				cout << "Before Out of bounds error" << endl;
				cout << "My = " << My[j] << "  dMy = " << dMy[j]<<endl;
			}
		}
		
		
		
		
		for (i=0;i<NumActin; i++){
				
			if (i >= Fixed_Actin){
				
				(ran2(idum) > 0.50) ? dAx[i] += D_step : dAx[i] -= D_step; // Diffusion Half-step
				Ax[i] += dAx[i];
				
				if (Ax[i] > (Lx - Lx_frame)) // Reflective BC
						Ax[i] -= 2*(Ax[i] - (Lx - Lx_frame));
				else if (Ax[i] < Lx_frame)
					Ax[i] += 2*(Lx_frame - Ax[i]);
			}
		}
		
		for (i=0;i<NumActin; i++){
			
			
			if (i >= Fixed_Actin){
			
				(ran2(idum) > 0.50) ? dAy[i] += D_step : dAy[i] -= D_step;
				Ay[i] += dAy[i];
				
				if (Ay[i] > (Ly - Ly_frame))
					Ay[i] -= 2*(Ay[i] - (Ly - Ly_frame));
				else if (Ay[i] < Ly_frame)
					Ay[i] += 2*(Ly_frame - Ay[i]);
			}
		}
		
		for (j=0;j<NumMyosin; j++){
			
			
			if (FreeM[j] == 1){
				(ran2(idum) > 0.50) ? dMx[j] += Dmu_f_step : dMx[j] -= Dmu_f_step;
				Mx[j] += dMx[j];
				if (Mx[j] > (Lx - Lx_frame))
					Mx[j] -= 2*(Mx[j] - (Lx - Lx_frame));
				else if (Mx[j] < Lx_frame)
					Mx[j] += 2*(Lx_frame - Mx[j]);
			}
			else {
				(ran2(idum) > 0.50) ? dMx[j] += Dmu_step : dMx[j] -= Dmu_step;
				Mx[j] += dMx[j];
				if (Mx[j] > (Lx - Lx_frame))
					Mx[j] -= 2*(Mx[j] - (Lx - Lx_frame));
				else if (Mx[j] < Lx_frame)
					Mx[j] += 2*(Lx_frame - Mx[j]);
			}

		}
		
		
		for (j=0;j<NumMyosin; j++){
			
			
			if (FreeM[j] == 1){
				(ran2(idum) > 0.50) ? dMy[j] += Dmu_f_step : dMy[j] -= Dmu_f_step;
				My[j] += dMy[j];
				if (My[j] > (Ly - Ly_frame))
					My[j] -= 2*(My[j] - (Ly - Ly_frame));
				else if (My[j] < Ly_frame)
					My[j] += 2*(Ly_frame - My[j]);
			}
			else {
				(ran2(idum) > 0.50) ? dMy[j] += Dmu_step : dMy[j] -= Dmu_step;
				My[j] += dMy[j];
				if (My[j] > (Ly - Ly_frame))
					My[j] -= 2*(My[j] - (Ly - Ly_frame));
				else if (My[j] < Ly_frame)
					My[j] += 2*(Ly_frame - My[j]);
			}

		}
		
		for (j=0; j<NumMyosin; j++) {
			if (Mx[j] < 0 || Mx[j] > Lx) {
				cout << "After Out of bounds error" << endl;
				cout << "Mx = " << Mx[j] << "  dMx = " << dMx[j]<<endl;
			}
			
			if (My[j] < 0 || My[j] > Ly) {
				cout << "After Out of bounds error" << endl;
				cout << "My = " << My[j] << "  dMy = " << dMy[j]<<endl;
			}
		}
		
		
		
		
		
		// Full 2D output
		//if( !(t/1000.0-t/1000) )
		if (!(t/1000.0-t/1000))
		{
			
			ofstream myfile;
			myfile.open("Dataout.m", ios::app);
			myfile.precision(6);
			/*
			 myfile<<"xs=[];"<<endl;
			 for(j=1; j<=NumXPoints; j+=1)
			 {
			 myfile<<"xs=[xs; ";
			 for(k=1; k<=NumYPoints; k+=1)
			 {
			 myfile<<(j-1)*dx<<" ";
			 }
			 myfile<<"];"<<endl;
			 }
			 
			 myfile<<"ys=[];"<<endl;
			 for(j=1; j<=NumXPoints; j+=1)
			 {
			 myfile<<"ys=[ys; ";
			 for(k=1; k<=NumYPoints; k+=1)
			 {
			 myfile<<(k-1)*dy<<" ";
			 }
			 myfile<<"];"<<endl;
			 }
			 
			 myfile<<"Phix=[];"<<endl;
			 for(j=1; j<=NumXPoints; j+=1)
			 {
			 myfile<<"Phix=[Phix; ";
			 for(k=1; k<=NumYPoints; k+=1)
			 {
			 myfile<<Phix[j][k]<<" ";
			 }
			 myfile<<"];"<<endl;
			 }
			 
			 myfile<<"Phiy=[];"<<endl;
			 for(j=1; j<=NumXPoints; j+=1)
			 {
			 myfile<<"Phiy=[Phiy; ";
			 for(k=1; k<=NumYPoints; k+=1)
			 {
			 myfile<<Phiy[j][k]<<" ";
			 }
			 myfile<<"];"<<endl;
			 }
			 */
			myfile<<"rho=[];"<<endl;
			for(j=1; j<=NumXPoints; j+=1)
			{
				myfile<<"rho=[rho; ";
				for(k=1; k<=NumYPoints; k+=1)
				{
					myfile<<rho[j][k]<<" ";
				}
				myfile<<"];"<<endl;
			}
			
			myfile<<"mu=[];"<<endl;
			for(j=1; j<=NumXPoints; j+=1)
			{
				myfile<<"mu=[mu; ";
				for(k=1; k<=NumYPoints; k+=1)
				{
					myfile<<mu[j][k]<<" ";
				}
				myfile<<"];"<<endl;
			}
			
			myfile<<"mu_f=[];"<<endl;
			for(j=1; j<=NumXPoints; j+=1)
			{
				myfile<<"mu_f=[mu_f; ";
				for(k=1; k<=NumYPoints; k+=1)
				{
					myfile<<mu_f[j][k]<<" ";
				}
				myfile<<"];"<<endl;
			}
			
			if (Frame_Count <= NumFrames){
				myfile<<"Rho_array(:, :, "<<Frame_Count<<") = rho;"<<endl;
				myfile<<"Mu_array(:, :, "<<Frame_Count<<") = mu;"<<endl;
				myfile<<"Mu_f_array(:, :, "<<Frame_Count<<") = mu_f;"<<endl;
				
				myfile<<"end_index = "<<Frame_Count<<";"<<endl;
				myfile<<"disp('Load of Data to to "<<Frame_Count<<" complete');"<<endl;
				/*myfile<<"figure(1), imagesc(rho), title(['Actin Density, time = ', num2str("<<t*dt<<")]);"<<endl;
				 myfile<<"figure(2), imagesc(mu), title(['Motor Density, time = ', num2str("<<t*dt<<")]);"<<endl;
				 myfile<<"Rho_movie(:,"<<Frame_Count<<") = getframe(fig1,winsize1);"<<endl;
				 myfile<<"Mu_movie(:,"<<Frame_Count<<") = getframe(fig2,winsize2);"<<endl;
				 myfile<<endl;*/
				Frame_Count++;
			}
			
			myfile.close();
			
			ofstream* file = new ofstream("Check.m");
			(*file)<<"check_mu=[];"<<endl;
			for(j=1; j<=NumXPoints; j+=1)
			{
				(*file)<<"check_mu=[check_mu; ";
				for(k=1; k<=NumYPoints; k+=1)
				{
					(*file)<<mu[j][k]<<" ";
				}
				(*file)<<"];"<<endl;
			}
			(*file) <<endl<<"figure(5), image(check_mu)";
			file -> close();
			delete file;
			
		}  
		
	}
	
	ofstream myfile;
	myfile.open("Dataout.m", ios::app);
	myfile.precision(6);
	myfile<<"disp('Load of File Complete!!!')"<<endl;
	myfile.close();
	
	return 0;
}
