#include <iostream>
#include <fstream>
#include <math.h>
using namespace std;

#include "accessories.h"
#include "auxlrs.h"

int main()
{
	//=========================================================================================================================
	//
	int i=0, j=0, k=0, t=0;
	
	/* GridData: an object holding an array 
	 implementing FFT
	 */
	
	const double D_step = sqrt(2*D*dt);
	const double Dmu_step = sqrt(2*Dmu*dt);
	const double Dmu_f_step = sqrt(2*Dmu_f*dt);
	

	
	double Ax[MaxNumActin]; // position of actin
	double Ay[MaxNumActin];
	
	double Mx[NumMyosin]; // position of myosin
	double My[NumMyosin];
	
	int rho_dett[NumXPoints][NumYPoints]; // 1 if decay occured in box, 0 if not
	
	int FreeM[NumMyosin]; // 0 if myosin is free 1 if its not free
	
	
	double P_d = 1 - exp(-nu_d*dt);
	double P_mu_att = 1 - exp(-mu_att*dt);
	double drho_for_N = 1/dx*1/dy;
	
	long * idum = new long;
	*(idum) = 100;
	
	GridData rho; // actin density
	GridData mu; // bound myosin density
	GridData mu_f; // free myosin, not used
	GridData T; // Total number of binding sites available for binding
	GridData muT; // an accessory object for evaluating the flux of myosin
	
	// corresponding fourier trasnforms
	GridData rho_hat;
	GridData mu_hat;
	GridData T_hat;
	GridData muT_hat;
	
	GridData drho;
	GridData dmu;
	GridData dmu_f;
	GridData dmuT;
	
	GridData Phix; // Force density
	GridData Phiy;
	
	GridData Jmux;
	GridData Jmuy;
	
	GridData J1x;
	GridData J1y;
	
	GridData J2x;
	GridData J2y;
	
	GridData P; // Probability distribution of motor arm displacement
	GridData fx; // force distribution, x-component
	GridData fy; // force distribution, y-component
	
	GridData P_hat;
	
	GridData Pfx;
	GridData Pfy;
	
	GridData Pfx_hat;
	GridData Pfy_hat;
	
	GridData xs; // Actual position of the grid
	GridData ys; // Actual position of the grid
	//=========================================================================================================================
	
	//=========================================================================================================================
	//

	for(i=1; i<=NumXPoints; i++)
	{
		for(j=1; j<=NumYPoints; j++)
		{
			xs[i][j] = (i-1)*dx;
			ys[i][j] = (j-1)*dy;
		}
	}
	
	// initialize distribution of motor arm displacement and force density
	for(i=1; i<=NumXPoints; i++)
	{
		for(j=1; j<=NumYPoints; j++)
		{
			double x = xs[i][j];
			if(x>Lx/2) x += -Lx;
			double y = ys[i][j];
			if(y>Ly/2) y += -Ly;
			
			P[i][j] = 1.0*exp( -(pow(x,2)+pow(y,2))/pow(5.0/8,2) );
			fx[i][j] = 100.0*x*exp( -(pow(x,2)+pow(y,2))/pow(5.0/8,2) );
			fy[i][j] = 100.0*y*exp( -(pow(x,2)+pow(y,2))/pow(5.0/8,2) );
			Pfx[i][j] = P[i][j]*fx[i][j];
			Pfy[i][j] = P[i][j]*fy[i][j];
		}
	}
	
	P_hat = &P;
	Pfx_hat = &Pfx;
	Pfy_hat = &Pfy;
	
	fft(&P_hat);
	fft(&Pfx_hat);
	fft(&Pfy_hat);
	//=========================================================================================================================
	
	//=========================================================================================================================
	// Initial conditions
	for (i=0; i<MaxNumActin; i++) {
		Ax[i] = -1;
		Ay[i] = -1;
	}
	
	for (i=0; i<NumActin; i++){
		Ax[i] = Lx*ran2(idum);
		Ay[i] = Ly*ran2(idum);
	}
	
	for (j=0; j<NumMyosin; j++){
		Mx[j] = Lx*ran2(idum);
		My[j] = Ly*ran2(idum);
		FreeM[j] = 1; // All myosin is initially free
	} 
	
	/*for (i=0; i<NumActin; i++){
		Ax[i] = fmod(i, 128)*dx + dx/2;
		Ay[i] = i/128*dy + dy/2;
	}
	
	
	for (i=0; i<NumXPoints; i++){
		for (j=0; j<NumYPoints; j++) {
			
			int tmp_id = i*128 + j;
			
			if (i < 64) {
				Mx[tmp_id] = Lx/2*sqrt(i/64.0) + dx/2;
			}
			else {
				Mx[tmp_id] = Lx - Lx/2*sqrt((128 - i)/64.0) + dx/2;
			}
			
			if (j < 64) {
				My[tmp_id] = Ly/2*sqrt(j/64.0) + dy/2;
			}
			else {
				My[tmp_id] = Ly - Ly/2*sqrt((128 - j)/64.0) + dy/2;
			}
		}
	}
	
	for (j=0; j<NumMyosin; j++){
		FreeM[j] = 1; // All myosin is initially bound
	} */
	//=========================================================================================================================
	
	//create the file for output
	
	ofstream* file = new ofstream("Dataout.m");
	(*file)<<"Lx = "<<Lx<<";"<<endl;
	(*file)<<"Ly = "<<Ly<<";"<<endl;
	(*file)<<"NumPointsX = "<<NumXPoints<<";"<<endl;
	(*file)<<"NumPointsY = "<<NumYPoints<<";"<<endl;
	(*file)<<"D = "<<D<<";"<<endl;
	(*file)<<"Alpha = "<<alpha<<";"<<endl;
	(*file)<<"Beta = "<<nu_d<<";"<<endl;
	(*file)<<"dx = "<<dx<<";"<<endl;
	(*file)<<"dy = "<<dy<<";"<<endl;
	(*file)<<"dt = "<<dt<<";"<<endl;
	
	(*file)<<"Rho_array = zeros("<<NumXPoints<<", "<<NumYPoints<<", "<<NumFrames<<");"<<endl;
	(*file)<<"Mu_array = zeros("<<NumXPoints<<", "<<NumYPoints<<", "<<NumFrames<<");"<<endl;
	
	(*file)<<"Time_Array = [];"<<endl;
	
	/*(*file)<<"mu=[];"<<endl;
	for(j=1; j<=NumXPoints; j+=1)
	{
		(*file)<<"mu=[mu; ";
		for(k=1; k<=NumYPoints; k+=1)
		{
			(*file)<<mu[j][k]<<" ";
		}
		(*file)<<"];"<<endl; 
	}*/
	
	file->close();
	delete file;
	
	//return 0;
	
	
	//=========================================================================================================================
	//=========================================================================================================================
	
	for(t=0; t < 101001;t++)
	{
		if( !(t/100.0-t/100) ) { cout<<"Iteration # "<<t<<" Time = "<<t*dt<<endl;
		}
		
		
		
		// initialize the density profiles to 0
		for (i=1; i<=NumXPoints; i++) {
			for (j=1; j<=NumYPoints; j++) {
				rho[i][j] = drho_for_N/10; // min
				mu[i][j] = drho_for_N/10; // min
				rho_dett[i-1][j-1] = 0;
			}
		}
		
		
		// create the density profile for actin
		for (i=0; i<NumActin; i++)
			rho[int(Ax[i]/dx) + 1][int(Ay[i]/dy) + 1] += drho_for_N;
		
		/*for (j=0; j<NumMyosin; j++)
			mu[int(Mx[j]/dx) + 1][int(My[j]/dy) + 1] += 1; */

		
		for (i=0;Ax[i] != -1; i++)
		{
			if (ran2(idum) < P_d)
			{
				// If this actin decays
				rho[int(Ax[i]/dx) + 1][int(Ay[i]/dy) + 1] -= drho_for_N; // remove this actin from the density
				rho_dett[int(Ax[i]/dx)][int(Ay[i]/dy)] = 1; // set flag that element decayed
				Ax[i] = Ax[NumActin-1]; // remove this position and replace it with substitute
				Ay[i] = Ay[NumActin-1];
				Ax[NumActin-1] = -1; // set the substitute to -1
				Ay[NumActin-1] = -1;
				NumActin--; // lower actin number by 1
				i--; // lower count by 1
			}
		
		}
		
		// Create New Actin
		for (i=0; i<AvgNumActin; i++) {
			if (ran2(idum) < P_d) { // if an element is created
				Ax[NumActin] = ran2(idum)*Lx; // generate a random position
				Ay[NumActin] = ran2(idum)*Ly;
				rho[int(Ax[NumActin]/dx) + 1][int(Ay[NumActin]/dy) + 1] += drho_for_N; // update the density profile
				NumActin++; // increase the number of actin by 1
			}
		} 
		
		//cout << "The Number of Actin is " << NumActin << endl;
			
		
		// create the density profile for myosin 
		for (j=0; j<NumMyosin; j++)
		{
			if (rho_dett[int(Mx[j]/dx)][int(My[j]/dy)] > 0) // if a dettachemnt occured in this spot
				if (ran2(idum) < 1/((rho[int(Mx[j]/dx) + 1][int(My[j]/dy) + 1]- drho_for_N/10)/drho_for_N + 1)) // if myosin dettached
					FreeM[j] = 1; // Free this myosin
		
			// If myosin dettached and there is actin present
			if ((FreeM[j] > 0)&&(rho[int(Mx[j]/dx) + 1][int(My[j]/dy) + 1] > drho_for_N/10))
				if (ran2(idum) < ((rho[int(Mx[j]/dx) + 1][int(My[j]/dy) + 1]- drho_for_N/10)/drho_for_N + 1)*P_mu_att) // if it attached
					FreeM[j] = 0; // make myosin bound
			
			if (FreeM[j] == 0) // Count only the myosin that is bound
				mu[int(Mx[j]/dx) + 1][int(My[j]/dy) + 1] += drho_for_N;
		} 
		
	
		 
		
		
		// Take the fourier transform of the density profiles
		// for both Actin and Myosin
		rho_hat = &rho; // Actin
		mu_hat = &mu; // Myosin
		fft(&rho_hat); // fft: fast Fourier transform
		fft(&mu_hat);
		
		convFourier(&P_hat,&rho_hat,&T_hat); // Convolve P with rho in Fourier space
		T = &T_hat;
		invfft(&T); // Invert Fourier transform
		
		for(j=1; j<=NumXPoints; j++)
		{
			for(k=1; k<=NumYPoints; k++)
			{
				muT[j][k] = mu[j][k]/(T[j][k]+eps);
			}
		}
		muT_hat = &muT;
		fft(&muT_hat);
		
		convFourier(&Pfx_hat,&rho_hat,&J1x);
		convFourier(&Pfy_hat,&rho_hat,&J1y);
		
		convFourier(&Pfx_hat,&muT_hat,&J2x);
		convFourier(&Pfy_hat,&muT_hat,&J2y);
		
		invfft(&J1x);
		invfft(&J1y);
		invfft(&J2x);
		invfft(&J2y);
		
		for(j=1; j<=NumXPoints; j++)
		{
			for(k=1; k<=NumYPoints; k++)
			{
				J1x[j][k] *= -mu[j][k]/(eps+T[j][k])/(eps+rho[j][k]); // add epsilon to take care of T=0
				J1y[j][k] *= -mu[j][k]/(eps+T[j][k])/(eps+rho[j][k]);
				
				J2x[j][k] *= -1.0;
				J2y[j][k] *= -1.0;
			}
		}
		
		// Finally calculate the force
		for(j=1; j<=NumXPoints; j++)
		{
			for(k=1; k<=NumYPoints; k++)
			{
				Phix[j][k] = alpha*(J1x[j][k]+J2x[j][k]);
				Phiy[j][k] = alpha*(J1y[j][k]+J2y[j][k]);

			}
		}
		
		/*
		// Move the particles
		double ***tmpvarx;
		double ***tmpvary;
		
		
		tmpvarx = Phix.get_f();
		tmpvary = Phiy.get_f();
		
		
		
		for (i=0; i<NumActin; i++) {
			int Tmpx = Ax[i]/dx;
			int Tmpy = Ay[i]/dy;
			
			Ax[i] += tmpvarx[1][Tmpx + 1][Tmpy + 1]*dx*dy;
			Ay[i] += tmpvary[1][Tmpx + 1][Tmpy + 1]*dx*dy;
		} 
		
		for (j=0; j<NumMyosin; j++){
			int Tmpx = Mx[j]/dx;
			int Tmpy = My[j]/dy;

			Mx[j] += tmpvarx[1][Tmpx + 1][Tmpy + 1]*dx*dy;
			My[j] += tmpvary[1][Tmpx + 1][Tmpy + 1]*dx*dy;
		} 
		 */
		
		for (i=0; i<NumActin; i++)
		{
			int Tmpx = Ax[i]/dx;
			int Tmpy = Ay[i]/dy;
			
			Ax[i]+=Phix[Tmpx + 1][Tmpy + 1]*dt;
			Ay[i]+=Phiy[Tmpx + 1][Tmpy + 1]*dt;
		}
		
		for (j=0; j<NumMyosin; j++) 
		{
			int Tmpx = Mx[j]/dx;
			int Tmpy = My[j]/dy;
			
			Mx[j]+=Phix[Tmpx + 1][Tmpy + 1]*dt;
			My[j]+=Phiy[Tmpx + 1][Tmpy + 1]*dt;
			
			/*if (Mx[j] > Lx) {
				cout << Phix[Tmpx + 1][Tmpy + 1]<<endl;
				cout << Mx[j];
				return 0;
			}
			if (Mx[j] < 0) {
				cout << Phix[Tmpx + 1][Tmpy + 1]<<endl;
				cout << Mx[j];
				return 0;
			}
			if (My[j] > Ly) {
				cout << Phix[Tmpx + 1][Tmpy + 1]<<endl;
				cout << My[j];
				return 0;
			}
			if (My[j] < 0) {
				cout << Phix[Tmpx + 1][Tmpy + 1]<<endl;
				cout << My[j];
				return 0;
			} */
			
		}
		
		 
		// Move by diffusing halfstep and implement periodic Bdy condition
		
		
		for (i=0;i<NumActin; i++){
			(ran2(idum) > 0.50) ? Ax[i] += D_step : Ax[i] -= D_step; // Diffusion Half-step
			if (Ax[i] > Lx) // Periodic BC
				Ax[i] -= Lx;
			else if (Ax[i] < 0)
				Ax[i] += Lx;
		}
		
		for (i=0;i<NumActin; i++){
			(ran2(idum) > 0.50) ? Ay[i] += D_step : Ay[i] -= D_step;
			if (Ay[i] > Ly)
				Ay[i] -= Ly;
			else if (Ay[i] < 0)
				Ay[i] += Ly;
		}
		
		for (j=0;j<NumMyosin; j++){
			
			if (FreeM[j] == 1) {
				(ran2(idum) > 0.50) ? Mx[j] += Dmu_f_step : Mx[j] -= Dmu_f_step;
				if (Mx[j] > Lx)
					Mx[j] -=Lx;
				else if (Mx[j] < 0)
					Mx[j] += Lx;
			}
			else {
				(ran2(idum) > 0.50) ? Mx[j] += Dmu_step : Mx[j] -= Dmu_step;
				if (Mx[j] > Lx)
					Mx[j] -=Lx;
				else if (Mx[j] < 0)
					Mx[j] += Lx;
			}
		}
		
		for (j=0;j<NumMyosin; j++){
			
			if (FreeM[j] == 1){
				(ran2(idum) > 0.50) ? My[j] += Dmu_f_step : My[j] -= Dmu_f_step;
				if (My[j] > Ly)
					My[j] -=Ly;
				else if (My[j] < 0)
					My[j] += Ly;
			}
			else {
				(ran2(idum) > 0.50) ? My[j] += Dmu_step : My[j] -= Dmu_step;
				if (My[j] > Ly)
					My[j] -=Ly;
				else if (My[j] < 0)
					My[j] += Ly;
			}
			
		}
		
		
			
		// Full 2D output
		//if( !(t/1000.0-t/1000) )
		if (!(t/1000.0-t/1000))
		{
			
			ofstream myfile;
			myfile.open("Dataout.m", ios::app);
			myfile.precision(6);
			/*
			myfile<<"xs=[];"<<endl;
			for(j=1; j<=NumXPoints; j+=1)
			{
				myfile<<"xs=[xs; ";
				for(k=1; k<=NumYPoints; k+=1)
				{
					myfile<<(j-1)*dx<<" ";
				}
				myfile<<"];"<<endl;
			}
			
			myfile<<"ys=[];"<<endl;
			for(j=1; j<=NumXPoints; j+=1)
			{
				myfile<<"ys=[ys; ";
				for(k=1; k<=NumYPoints; k+=1)
				{
					myfile<<(k-1)*dy<<" ";
				}
				myfile<<"];"<<endl;
			}
			
			myfile<<"Phix=[];"<<endl;
			for(j=1; j<=NumXPoints; j+=1)
			{
				myfile<<"Phix=[Phix; ";
				for(k=1; k<=NumYPoints; k+=1)
				{
					myfile<<Phix[j][k]<<" ";
				}
				myfile<<"];"<<endl;
			}
			
			myfile<<"Phiy=[];"<<endl;
			for(j=1; j<=NumXPoints; j+=1)
			{
				myfile<<"Phiy=[Phiy; ";
				for(k=1; k<=NumYPoints; k+=1)
				{
					myfile<<Phiy[j][k]<<" ";
				}
				myfile<<"];"<<endl;
			}
			*/
			
			myfile<<"Time_Array = [Time_Array;"<<t<<"  ];"<<endl;
			
			/*myfile<<"Fx=[];"<<endl;
			for(j=1; j<=NumXPoints; j+=1)
			{
				myfile<<"Fx=[Fx; ";
				for(k=1; k<=NumYPoints; k+=1)
				{
					myfile<<Phix[j][k]<<" ";
				}
				myfile<<"];"<<endl;
			}
			
			myfile<<"Fy=[];"<<endl;
			for(j=1; j<=NumXPoints; j+=1)
			{
				myfile<<"Fy=[Fy; ";
				for(k=1; k<=NumYPoints; k+=1)
				{
					myfile<<Phiy[j][k]<<" ";
				}
				myfile<<"];"<<endl;
			}
			 */
			
			myfile<<"rho=[];"<<endl;
			for(j=1; j<=NumXPoints; j+=1)
			{
				myfile<<"rho=[rho; ";
				for(k=1; k<=NumYPoints; k+=1)
				{
					myfile<<rho[j][k]<<" ";
				}
				myfile<<"];"<<endl;
			}
			
			myfile<<"mu=[];"<<endl;
			for(j=1; j<=NumXPoints; j+=1)
			{
				myfile<<"mu=[mu; ";
				for(k=1; k<=NumYPoints; k+=1)
				{
					myfile<<mu[j][k]<<" ";
				}
				myfile<<"];"<<endl;
			}
			
			if (Frame_Count <= NumFrames){
				myfile<<"Rho_array(:, :, "<<Frame_Count<<") = rho;"<<endl;
				myfile<<"Mu_array(:, :, "<<Frame_Count<<") = mu;"<<endl;
				/*myfile<<"figure(1), imagesc(rho), title(['Actin Density, time = ', num2str("<<t*dt<<")]);"<<endl;
				myfile<<"figure(2), imagesc(mu), title(['Motor Density, time = ', num2str("<<t*dt<<")]);"<<endl;
				myfile<<"Rho_movie(:,"<<Frame_Count<<") = getframe(fig1,winsize1);"<<endl;
				myfile<<"Mu_movie(:,"<<Frame_Count<<") = getframe(fig2,winsize2);"<<endl;
				myfile<<endl;*/
				Frame_Count++;
			}
			
			myfile.close();

		}  
		
	}
	
	return 0;
}
