/* $Revision: 1.8.6.2 $ */
/*=========================================================
 * convec.c
 * example for illustrating how to use pass complex data 
 * from MATLAB to C and back again
 *
 * convolves  two complex input vectors
 *
 * This is a MEX-file for MATLAB.
 * Copyright 1984-2006 The MathWorks, Inc.
 *=======================================================*/
#include "mex.h"

/* computational subroutine */
void Move_MoleculesC( double *Mpos, double *Apos, double *Mforce, double *Aforce,
             double *Mdrag, double *Adrag, double *AttachM, double *Springlen, double *dt,
             mwSize nMpos, mwSize nApos,
             double *NewMpos, double *NewApos, double *Newspringlen)
{
    mwSize j;
    
    /* Move the Actin*/
    *(NewApos) = *(Apos) + *(dt)* *(Aforce)/ *(Adrag);
    *(NewApos+1) = *(Apos+1) + *(dt)* *(Aforce)/ * (Adrag);
    
    /* Move the Myosin and change the new spring length*/
    for (j=0; j<nMpos; j++){

        *(NewMpos+j) = *(Mpos+j) + *(dt)* *(Mforce+j)/ *(Mdrag);

        if ( *(AttachM+j) > 0){
           *(Newspringlen+j) = *(Springlen+j) - *(dt)* *(Mforce+j)/ *(Mdrag) + *(dt)* *(Aforce)/ *(Adrag);
        }
        else{
            *(Newspringlen+j) = *(Springlen+j);
        }
       if ( *(AttachM+nMpos + j) > 0){
           *(Newspringlen+nMpos+j) = *(Springlen+nMpos+j) - *(dt)* *(Mforce+j)/ *(Mdrag) + *(dt)* *(Aforce)/ *(Adrag);
       }
       else{
            *(Newspringlen+nMpos+j) = *(Springlen+nMpos+j);
       }
    }
}

/* The gateway routine. */
void mexFunction( int nlhs, mxArray *plhs[],
                  int nrhs, const mxArray *prhs[] )
{
    double  *Mpos, *Apos, *Mforce, *Aforce, *Mdrag, *Adrag, *AttachM, *Springlen, *dt, *NewMpos, *NewApos, *Newspringlen;
    mwSize  nMpos, nApos;
    
    /* check for the proper number of ar
    /* get the length of Mpos and Apos. Everything else following here will 
     require only these two indices*/
    nMpos = mxGetM(prhs[0]);
    nApos = mxGetM(prhs[1]);


    /* get pointers to the input */
    Mpos = mxGetPr(prhs[0]);
    Apos = mxGetPr(prhs[1]);
    Mforce = mxGetPr(prhs[2]);
    Aforce = mxGetPr(prhs[3]);
    Mdrag = mxGetPr(prhs[4]);
    Adrag = mxGetPr(prhs[5]);
    AttachM = mxGetPr(prhs[6]);
    Springlen = mxGetPr(prhs[7]);
    dt = mxGetPr(prhs[8]);
    
    /* create new arrays for the output and set the output pointers to them */
    plhs[0] = mxCreateDoubleMatrix(nMpos, 1, mxREAL);
    plhs[1] = mxCreateDoubleMatrix(nApos, 2, mxREAL);
    plhs[2] = mxCreateDoubleMatrix(nMpos, 2, mxREAL);
    NewMpos = mxGetPr(plhs[0]);
    NewApos = mxGetPr(plhs[1]);
    Newspringlen = mxGetPr(plhs[2]);

    /* call the C subroutine */
    Move_MoleculesC(Mpos, Apos, Mforce, Aforce, Mdrag, Adrag, AttachM, Springlen, dt, nMpos, nApos, NewMpos, NewApos, Newspringlen);

    return;
}




