function [New_Myosin, New_Actin, New_Props] = Move(Myosin, Actin, Props)
%MOVE This function moves the constituents and then sees if any myosin
%moved into the ends region of the actin

New_Myosin = Myosin;
New_Actin = Actin;
New_Props = Props;

NDragVector = max(Props.drag_cluster, [], 1) + ...
    Myosin.drag*(max(Props.drag_cluster, [], 1) == 0);


ADragVector = max(Props.drag_cluster, [], 2)' + ...
    Actin.drag*(max(Props.drag_cluster, [], 2) == 0)';
    

%%{
%first move all constituents that are NOT in a cluster
%Move the unclustered Myosin
Unclus_N = (NDragVector == Myosin.drag); %Pick the ones that are not in cluster
Unclus_N_Conn = Props.connections(:, Unclus_N); %Find their connections
%Find the distributed step
tempN = sum(Unclus_N_Conn, 1)./sum(abs(Unclus_N_Conn), 1); 
tempN(isnan(tempN)) = 0;

%Diffusion step for Myosin
Nodes_D_step = Props.random_force/Myosin.drag*(2*floor(2*rand(sum(Unclus_N), 1)) - 1); 

%Move the Myosin
New_Myosin.pos(Unclus_N) = Myosin.pos(Unclus_N) + Myosin.force*(1./Myosin.drag)*tempN' + Nodes_D_step;
%%}
    
%%{    
%Move the unclustered Actin
Unclus_A = (ADragVector == Actin.drag); %The unclustered actin
Unclus_A_Conn = Props.connections(Unclus_A, :); %Its connections
%Total amount of attached
tempA = sum(abs(Unclus_A_Conn), 2);
temp2A = tempA;
temp2A(tempA > 0) = 1./(tempA(tempA > 0));
FiberForce = sum(Unclus_A_Conn.*repmat(temp2A, 1, Myosin.num), 2);


%Diffusion step for Actin
Actin_D_step = Props.random_force/Actin.drag*(2*floor(2*rand(sum(Unclus_A), 1)) - 1);

New_Actin.pos(Unclus_A, 1) = Actin.pos(Unclus_A, 1) - Myosin.force*(1/Actin.drag)*FiberForce ...
    + Actin_D_step;

New_Actin.pos(Unclus_A, 2) = Actin.pos(Unclus_A, 1) ...
    - Actin.length(Unclus_A) .* Actin.polarization(Unclus_A);
%}

FiberForce = sum(Props.connections.*repmat(sum(abs(Props.connections), 1), Actin.num, 1), 2);
% Move the clusters
for j=1:max(max(Props.cluster)),
    [Fs, Ns] = find(Props.cluster == j);
    Ns = sort(Ns); Fs = sort(Fs);
    FiltN = [1;(Ns(2:end) - Ns(1:end-1))];
    Nclus = Ns(FiltN ~=0); %The index of the nodes that are in cluster j
    
    %ClusPos = mean(Nodes(Nclus));
    
    FiltF = [1;(Fs(2:end) - Fs(1:end-1))];
    Fclus = Fs(FiltF ~= 0); %The index of the fibers that are in cluster j
    
    ClusterDrag = max(NDragVector(Nclus));
    
    ClusterDstep = Props.random_force/ClusterDrag * (2*floor(2*rand(1)) - 1);
    
    ClusterForce = sum(Myosin.force*(1./NDragVector(Nclus))'.*mean(Props.connections(:, Nclus), 1)') ...
        - sum(Myosin.force*(1./ADragVector(Fclus))'.*FiberForce(Fclus));
    
    
    New_Myosin.pos(Nclus) = Myosin.pos(Nclus) + ClusterForce + ClusterDstep;
    New_Actin.pos(Fclus, 1) = Actin.pos(Fclus, 1) + ClusterForce + ClusterDstep;
    New_Actin.pos(Fclus, 2) = New_Actin.pos(Fclus, 1) ...
        - Actin.length(Fclus) .* Actin.polarization(Fclus);
end

[New_Props.connections_ends] = Identify_connections_ends(New_Myosin, Myosin, Actin, New_Actin, Props);


end

