%%Simulation of the actin and mysoin in 1D. 
clear all;
close all;
clc;
%% Set up initial parameters
% Number of Constituents

M.num = 50; %number of myosin nodes
A.num = 50; %number of actin fibers

% Time of Simulation
P.dt = .1;
P.Tend = 2000;
P.Tdisplay = 10;
Position_Data.TimeVector = linspace(0, P.Tend, P.dt);

% Fiber Characteristics
A.fraction_pol = .5;%the faction of the actin that is polarized in the -X direction
A.average_length = 15; %the average length of the actin fibers
A.length_std = 3; %the standard deviation of the length of the fibers

% Myosin/Actin Dymanics Characteristics
A.t_half  = 1000; %Half life of the connections at the ends

P.random_force = 0; %Diffusion: Random step in time dt

%M.capture_radius = .2;%the capture radius of the myosin node
M.drag = 1; %drag of the myosin
A.drag = 1; %drag of the actin
A.end_size = .2; %the size of the end region
M.force = .1; %force between one actin and myosin. If one myosin rests on an actin fiber then it will walk force/drag*t in time t


% Linear Space Charactetistics
P.length = 100; %the radius of the ring

% Display Titles
%ParDisplay = [' Thalf = ', num2str(Thalf), ' Rforce = ', num2str(RandomForce), ' Mforce = ', num2str(MyosinForce)];
%MovieTitle = [num2str(n), 'N', num2str(f), 'F, ', num2str(Mdrag), 'Md', num2str(Adrag), 'Ad' , ', RF = ', num2str(RandomForce), ', MF = ', num2str(MyosinForce), ', Thalf = ', num2str(Thalf)];
%ImageTitle = ['Mdrag = ', num2str(Mdrag), ' A drag = ', num2str(Adrag), ' MF = ', num2str(MyosinForce), ' Thalf = ', num2str(Thalf)];
FileName = ('Test.bin');

% Generating the positions of the Nodes and Fibers and Fiber Polatization
M.pos = linspace(0, P.length, M.num)';

%M.pos = 50 * ones(M.num, 1);

%Select Actin Polarization
%note: if A.pos(i, 1) > A.pos(i, 2) then fiber is polarized in the
%+X direction. Otherwise it is polarized in the -X direction
%Note: The polarized end of the fiber is always located on the ring
A.polarization = ones(A.num,1);
A.polarization(1 : floor(A.num*A.fraction_pol)) = -1;

%Select Actin length
A.length = normrnd(A.average_length, A.length_std, A.num, 1);

%Select Actin initial position
A.pos = ones(A.num, 2) + 1;
%A.pos(:, 1) = linspace(0, P.length, A.num)';
%A.pos(:, 1) = 100.01;
A.pos(:, 1) = P.length * rand(A.num, 1);


A.pos(:, 2) = A.pos(:, 1) - A.length .* A.polarization;

% Dymanics Matricies
P.connections = zeros(A.num, M.num); %What nodes are interacting with what fibers
P.connections_ends = zeros(A.num, M.num); %What nodes are connected to the END of what fiber
P.cluster = zeros(A.num, M.num); %The structure matrix
P.drag_cluster = zeros(A.num, M.num); %The cluster matrix with its total drag

Display_all(M.pos, A.pos)


Position_Data.Myosin = M.pos;
Position_Data.Actin = A.pos;
Position_Data.polarization = A.polarization;

%% Movie Display
%Display_all_ring(Nodes_pos, Fiber_pos, Polarization, Rring), axis([0 2*pi*Rring -.5 20.5]), title(0);
%nframes = Tend/Tdisplay; %number of frames
%Movie = moviein(nframes);

%nf = 1;

%% Simulation
for t = 0:P.dt:P.Tend
    %% Identify the Connections between myosin and actin
    P.connections = Indentify_connections(M, A);
    
    % Move the filaments and determine which moved into the ends region
    [M, A, P] = Move(M, A, P);
    drawnow;
    pause(.1);
    Display_all(M.pos, A.pos)
    %% Decay the connections
    [P.connections_ends] = Connection_turnover(A, P);
    
    %% Identify the structure built at ends
    P.cluster = Find_Conn(P.connections_ends);
    
    %%  Assing the drag to the clusters
    P.drag_cluster = Assing_Drag(A, M, P);

    %% Record the positions of the constituents
    if mod(t, P.Tdisplay) == 0,
        pause(.01);
        Position_Data.Myosin = [Position_Data.Myosin, M.pos];
        Position_Data.Actin = [Position_Data.Actin, A.pos];
    end

end  
    
save(FileName, 'Position_Data', 'M', 'A', 'P');

